# Script to run all sample programs

use Cwd;

select STDERR; $| = 1;
select STDOUT; $| = 1;

my $windows = ($^O =~ /Win/);
my $cwd = cwd(); chomp($cwd);
my $makeprog = ($windows) ? 'nmake -nologo' : 'make';
my $arg = shift;

# Set PATH (Windows) or LD_LIBRARY_PATH (Linux) env var

my $dr = &debugOrRelease();
if ($windows) {
    $ENV{'PATH'} .= ";$cwd\\$dr\\lib";
}
else {
    $ENV{'LD_LIBRARY_PATH'} = "$cwd/$dr/lib";
}

# Loop through sample directories and run programs

my @sampleGroups = ("test_ts24008_msgs", "test_ts24301_msgs", 
                     "test_ts44018_msgs", "test_ts24501_msgs");
foreach $group (@sampleGroups) {
   chdir ("./$group");
   opendir (DIR, ".");
   my @sampleDirs = readdir (DIR);
   closedir (DIR);
   
   foreach $dir (@sampleDirs) {
       next if ($dir eq '.' || $dir eq '..');
       if (-d $dir) {
           print ("$group/$dir\n");
           chdir ("./$dir") || die ("chdir $dir failed");
   
           if ($arg eq "-clean") {
               `$makeprog clean`;
           }
           else {
               `$makeprog`;
   
               if (-e "writer.exe" || -e "writer") {
                   print ("executing writer ..\n");
                   $ret = system ("writer -v > writer.log") if ($windows);
                   $ret = system ("./writer -v > writer.log") if (!$windows);
                   if ($ret) {
                       $ret >>= 8;
                       ($ret & 0x80) ? -(0x100 - ($ret & 0xFF)) : $ret;
                       print ("writer failed! stat = $ret, check writer.log\n");
                       chdir ('..');
                       next;
                   }
                   if (-e "./good/writer.log") {
                       print "diff writer.log ./good/writer.log\n";
                       print `diff writer.log ./good/writer.log`;
                   }
                   print ("writer complete\n");
               }
   
               if (-e "reader.exe" || -e "reader") {
                   print ("executing reader ..\n");
                   $ret = system ("reader -v > reader.log") if ($windows);
                   $ret = system ("./reader -v > reader.log" ) if (!$windows);
                   if ($ret) {
                       $ret >>= 8;
                       ($ret & 0x80) ? -(0x100 - ($ret & 0xFF)) : $ret;
                       print ("reader failed! stat = $ret, check reader.log\n");
                       chdir ('..');
                       next;
                   }
                   if (-e "./good/reader.log") {
                       print "diff reader.log ./good/reader.log\n";
                       print `diff reader.log ./good/reader.log`;
                   }
                   print ("reader complete\n");
                   print ("***\n\n");
               }
           }
   
           chdir ('..');
       }
   }
   
   chdir ('..');
}

sub debugOrRelease() {
   die ("No plaform.mk file.") if (not -e "platform.mk");
   open (PLATFORM_MK, "platform.mk");
   while (<PLATFORM_MK>) {
      if (index ($_, "START DEBUG") != -1) {
         $nextLine = <PLATFORM_MK>;
         close (PLATFORM_MK);
         if ($nextLine =~ /^\#/) {
            return "release";
         }
         else {
            return "debug";
         }
      }
      if (/^BLDSUBDIR/) {
         @pieces = split (/=/, $_);
         $value = $pieces[1];
         chomp ($value);
         $value =~ s/^\s+//;
         $value =~ s/\s+$//;
         close (PLATFORM_MK);
         return $value;
      }
   }
   # If we couldn't find anything definite, assume debug.
   close (PLATFORM_MK);
   return "debug";
}
