#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_ServRequest data;
   TS24501Msg_ServRequest* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */
   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */
   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 1;
   pdu.secHdr.protoDiscr = TS24501Msg_NAS5GProtoDiscr_mobMgmt5G;
   pdu.secHdr.secHdrType = TS24501Msg_NAS5GSecHdrType_integProt;
   pdu.secHdr.msgAuthCode = 1000;
   pdu.secHdr.seqNumber  = 100;
   pdu.hdrData.u.mm.secHdrType = TS24501Msg_NAS5GSecHdrType_noSec;

   asn1SetTC_TS24501Msg_PDU_obj_ServRequest5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_ServRequest (pvalue);

   pvalue->serviceType = TS24501IE_ServiceType_emerSvc;

   pvalue->nasKeySetIdent.tsc = 0;
   pvalue->nasKeySetIdent.ident = 2;

   pvalue->stmsi.t = T_TS24501IE_MobileIdentity_stmsi;
   pvalue->stmsi.u.stmsi.amfSetID = 13;
   pvalue->stmsi.u.stmsi.amfPointer = 8;
   {
      static const OSOCTET testdata[] = { 
      0xe3, 0x42, 0x47, 0xa0 };
      OSCRTLSAFEMEMCPY ((void*)pvalue->stmsi.u.stmsi.tmsi, 4, testdata, 4);
   }

   pvalue->m.uplinkStatPresent = 1;
   pvalue->uplinkStat.length = 0; /* length will be calculated */
   pvalue->uplinkStat.length = 0; /* length will be calculated */
   pvalue->uplinkStat.psi_3 = TRUE;
   pvalue->uplinkStat.psi_0 = TRUE;
   pvalue->uplinkStat.psi_13 = TRUE;
   pvalue->uplinkStat.psi_1 = FALSE;
   pvalue->uplinkStat.psi_2 = FALSE;
   pvalue->uplinkStat.psi_4 = FALSE;
   pvalue->uplinkStat.psi_5 = FALSE;
   pvalue->uplinkStat.psi_6 = FALSE;
   pvalue->uplinkStat.psi_7 = FALSE;
   pvalue->uplinkStat.psi_8 = FALSE;
   pvalue->uplinkStat.psi_9 = FALSE;
   pvalue->uplinkStat.psi_10 = FALSE;
   pvalue->uplinkStat.psi_11 = FALSE;
   pvalue->uplinkStat.psi_12 = FALSE;
   pvalue->uplinkStat.psi_14 = FALSE;
   pvalue->uplinkStat.psi_15 = FALSE;

   pvalue->m.sessionStatPresent = 1;
   pvalue->sessionStat.length = 0; /* length will be calculated */
   pvalue->sessionStat.length = 10;
   pvalue->sessionStat.psi_7 = TRUE;
   pvalue->sessionStat.psi_2 = TRUE;
   pvalue->sessionStat.psi_13 = TRUE;
   pvalue->sessionStat.psi_0 = FALSE;
   pvalue->sessionStat.psi_1 = FALSE;
   pvalue->sessionStat.psi_3 = FALSE;
   pvalue->sessionStat.psi_4 = FALSE;
   pvalue->sessionStat.psi_5 = FALSE;
   pvalue->sessionStat.psi_6 = FALSE;
   pvalue->sessionStat.psi_8 = FALSE;
   pvalue->sessionStat.psi_9 = FALSE;
   pvalue->sessionStat.psi_10 = FALSE;
   pvalue->sessionStat.psi_11 = FALSE;
   pvalue->sessionStat.psi_12 = FALSE;
   pvalue->sessionStat.psi_14 = FALSE;
   pvalue->sessionStat.psi_15 = FALSE;   

   pvalue->m._v17ExtPresent = 1;
   pvalue->m.ueRequestTypePresent = 1;
   pvalue->ueRequestType.length = 0;  // Calculated automatically...
   pvalue->ueRequestType.type = TS24301IE_UERequestTypes_ueNASRelease;

   pvalue->m.pagingRestrictionPresent = 1;
   pvalue->pagingRestriction.length = 0;  // Calculated automatically...
   pvalue->pagingRestriction.type = TS24301IE_RestrictionTypes_rtExceptVoicePDN;

   pvalue->pagingRestriction.m.octet4Present = 1;
   pvalue->pagingRestriction.octet4.ebi7 = 1;
   pvalue->pagingRestriction.octet4.ebi6 = 0;
   pvalue->pagingRestriction.octet4.ebi5 = 0;
   pvalue->pagingRestriction.octet4.ebi4 = 0;
   pvalue->pagingRestriction.octet4.ebi3 = 1;
   pvalue->pagingRestriction.octet4.ebi2 = 1;
   pvalue->pagingRestriction.octet4.ebi1 = 0;
   pvalue->pagingRestriction.octet4.ebi0 = 0;

   pvalue->pagingRestriction.m.octet5Present = 1;
   pvalue->pagingRestriction.octet5.ebi15 = 0;
   pvalue->pagingRestriction.octet5.ebi14 = 0;
   pvalue->pagingRestriction.octet5.ebi13 = 1;
   pvalue->pagingRestriction.octet5.ebi12 = 0;
   pvalue->pagingRestriction.octet5.ebi11 = 0;
   pvalue->pagingRestriction.octet5.ebi10 = 1;
   pvalue->pagingRestriction.octet5.ebi9 = 0;
   pvalue->pagingRestriction.octet5.ebi8 = 1;

   /* Enable bit tracing */
   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */
   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */
   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
